/*
 * Copyright 2007-2008 Ben Boeckel <MathStuf@gmail.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Header include
#include "Badge.h"

// Sigmod includes
#include "Macros.h"
#include "Rules.h"
#include "Sigmod.h"
#include "Sprite.h"

Sigmod::Badge::Badge(const Badge& badge) :
        Object(badge.parent(), badge.id()),
        m_stat(ST_End_GSC - ST_No_HP_Start)
{
    *this = badge;
}

Sigmod::Badge::Badge(const Sigmod* parent, const int id) :
        Object(parent, id),
        m_name(""),
        m_face(-1),
        m_badge(-1),
        m_obey(0),
        m_stat(ST_End_GSC - ST_No_HP_Start)
{
    for (int i = 0; i < m_stat.size(); ++i)
        m_stat[i].set(1, 1);
}

Sigmod::Badge::Badge(const Badge& badge, const Sigmod* parent, const int id) :
        Object(parent, id),
        m_stat(ST_End_GSC - ST_No_HP_Start)
{
    *this = badge;
}

Sigmod::Badge::Badge(const QDomElement& xml, const Sigmod* parent, const int id) :
        Object(parent, id),
        m_stat(ST_End_GSC - ST_No_HP_Start)
{
    LOAD_ID();
    load(xml);
}

void Sigmod::Badge::validate()
{
    TEST_BEGIN();
    if (m_name.isEmpty())
        emit(error("Name is empty"));
    TEST(setFace, face);
    TEST(setBadge, badge);
    TEST(setObey, obey);
    TEST_ARRAY(setStat, stat, ST_End_GSC);
    TEST_END();
}

void Sigmod::Badge::load(const QDomElement& xml)
{
    LOAD_BEGIN();
    LOAD(name);
    LOAD(face);
    LOAD(badge);
    LOAD(obey);
    LOAD_ARRAY(stat);
}

QDomElement Sigmod::Badge::save() const
{
    SAVE_CREATE();
    SAVE(name);
    SAVE(face);
    SAVE(badge);
    SAVE(obey);
    SAVE_ARRAY(stat);
    return xml;
}

void Sigmod::Badge::setName(const QString& name)
{
    CHECK(name);
}

void Sigmod::Badge::setFace(const int face)
{
    if (qobject_cast<const Sigmod*>(sigmod())->spriteIndex(face) == INT_MAX)
        emit(error(bounds("face")));
    else
        CHECK(face);
}

void Sigmod::Badge::setBadge(const int badge)
{
    if (qobject_cast<const Sigmod*>(sigmod())->spriteIndex(badge) == INT_MAX)
        emit(error(bounds("badge")));
    else
        CHECK(badge);
}

void Sigmod::Badge::setObey(const int obey)
{
    if (qobject_cast<const Sigmod*>(sigmod())->rules()->maxLevel() < obey)
    {
        emit(error(bounds("obey")));
        return;
    }
    CHECK(obey);
}

void Sigmod::Badge::setStat(const Stat stat, const Fraction& multiplier)
{
    if (multiplier < 1)
    {
        emit(error(bounds("stat multiplier")));
        return;
    }
    switch (stat)
    {
        case ST_SpecialDefense:
            if (qobject_cast<const Sigmod*>(sigmod())->rules()->specialSplit())
        case ST_Attack ... ST_Special:
            break;
        default:
            emit(warning(bounds("stat")));
            return;
    }
    CHECK_ARRAY(stat[stat - ST_No_HP_Start], multiplier);
}

void Sigmod::Badge::setStat(const Fraction& multiplier)
{
    if (multiplier < 1)
        emit(error(bounds("stat multiplier")));
}

QString Sigmod::Badge::name() const
{
    return m_name;
}

int Sigmod::Badge::face() const
{
    return m_face;
}

int Sigmod::Badge::badge() const
{
    return m_badge;
}

int Sigmod::Badge::obey() const
{
    return m_obey;
}

Sigmod::Fraction Sigmod::Badge::stat(const Stat stat) const
{
    switch (stat)
    {
        case ST_SpecialDefense:
            if (qobject_cast<const Sigmod*>(sigmod())->rules()->specialSplit())
        case ST_Attack ... ST_Special:
            return m_stat[stat - ST_No_HP_Start];
        default:
            emit(warning(bounds("stat")));
            return Fraction();
    }
}

Sigmod::Badge& Sigmod::Badge::operator=(const Badge& rhs)
{
    if (this == &rhs)
        return *this;
    COPY(name);
    COPY(face);
    COPY(badge);
    COPY(obey);
    COPY(stat);
    return *this;
}
