/*
 * Copyright 2008 Ben Boeckel <MathStuf@gmail.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SIGMODR_TYPECHARTMODEL
#define SIGMODR_TYPECHARTMODEL

// Sigmod includes
#include "../sigmod/Fraction.h"
#include "../sigmod/Matrix.h"

// KDE includes
#include <KColorScheme>

// Qt includes
#include <QtCore/QAbstractTableModel>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

namespace Sigmodr
{
class TypechartModel : public QAbstractTableModel
{
    Q_OBJECT
    
    public:
        TypechartModel(Sigmod::Matrix<Sigmod::Fraction>& typechart, const QStringList& types);
        
        QVariant data(const QModelIndex& index, int role) const;
        
        QVariant headerData(int section, Qt::Orientation /*orientation*/, int role = Qt::DisplayRole) const;
        
        int rowCount(const QModelIndex& /*parent*/) const;
        
        int columnCount(const QModelIndex& /*parent*/) const;
        
        Qt::ItemFlags flags(const QModelIndex& index) const;
        
        bool setData(const QModelIndex& index, const QVariant& value, int role);
        
        void discarded();
    private:
        Sigmod::Matrix<Sigmod::Fraction>& m_typechart;
        const QStringList m_types;
};

inline TypechartModel::TypechartModel(Sigmod::Matrix<Sigmod::Fraction>& typechart, const QStringList& types) :
        QAbstractTableModel(),
        m_typechart(typechart),
        m_types(types)
{
}

inline QVariant TypechartModel::data(const QModelIndex& index, int role) const
{
    if (!index.isValid())
        return QVariant();
    if (role == Qt::DisplayRole)
        return QString::number(m_typechart.at(index.row(), index.column()), 'g', 3);
    else if (role == Qt::EditRole)
        return QVariant::fromValue(m_typechart.at(index.row(), index.column()));
    else if (role == Qt::BackgroundRole)
    {
        double multiplier = m_typechart.at(index.row(), index.column());
        if (multiplier <= .5)
            return KStatefulBrush(KColorScheme::View, KColorScheme::NegativeBackground).brush(QPalette::Normal);
        else if (multiplier < 2.)
            return KStatefulBrush(KColorScheme::View, KColorScheme::NeutralBackground).brush(QPalette::Normal);
        else
            return KStatefulBrush(KColorScheme::View, KColorScheme::PositiveBackground).brush(QPalette::Normal);
    }
    return QVariant();
}

inline QVariant TypechartModel::headerData(int section, Qt::Orientation /*orientation*/, int role) const
{
    if (role == Qt::DisplayRole)
        return m_types.at(section);
    return QVariant();
}

inline int TypechartModel::rowCount(const QModelIndex& /*parent*/) const
{
    return m_typechart.height();
}

inline int TypechartModel::columnCount(const QModelIndex& /*parent*/) const
{
    return m_typechart.width();
}

inline Qt::ItemFlags TypechartModel::flags(const QModelIndex& index) const
{
    if (!index.isValid())
        return 0;
    return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
}

inline bool TypechartModel::setData(const QModelIndex& index, const QVariant& value, int role)
{
    if (!index.isValid())
        return false;
    if (role == Qt::EditRole)
    {
        if (value.canConvert<Sigmod::Fraction>())
        {
            m_typechart(index.row(), index.column()) = value.value<Sigmod::Fraction>();
            emit(dataChanged(index, index));
            return true;
        }
    }
    return false;
}

inline void TypechartModel::discarded()
{
    reset();
}

}

#endif
